/*
 * knights_engine.hpp
 *
 * This file is part of Knights.
 *
 * Copyright (C) Stephen Thompson, 2006 - 2009.
 * Copyright (C) Kalle Marjola, 1994.
 *
 * Knights is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Knights is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Knights.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

/*
 * This is the main class that runs a single game of Knights.
 *
 * NOTE: There is a limitation that only one instance of KnightsEngine
 * can be created per thread at the moment. This is because of the way
 * my Mediator class works.
 *
 */

#ifndef KNIGHTS_ENGINE_HPP
#define KNIGHTS_ENGINE_HPP

#include "boost/shared_ptr.hpp"

class DungeonView;
class KnightsCallbacks;
class KnightsConfig;
class KnightsEngineImpl;
class MenuSelections;
class MiniMap;
class StatusDisplay;
class UserControl;

class KnightsEngine {
public:
    // Start up a new KnightsEngine. Requires KnightsConfig and menu settings.
    // Note that the same KnightsConfig can be re-used between different KnightsEngines.
    KnightsEngine(boost::shared_ptr<const KnightsConfig> config,
                  const MenuSelections &msel,
                  int house_col_0, int house_col_1);
    ~KnightsEngine();

    // Run one update step (for a given time).
    // Uses KnightsCallbacks to inform caller of what happened during the update.
    void update(int time_delta, KnightsCallbacks &callbacks);

    // Input cmds that might be received from players.
    void setControl(int player, const UserControl *control);

    // Send initial updates to an observer who has just joined the game.
    void catchUp(int player, DungeonView &dungeon_view, MiniMap &mini_map, StatusDisplay &status_display);
    
private:
    boost::shared_ptr<KnightsEngineImpl> pimpl;
};

#endif
