/*
 * rng.cpp
 *
 * This file is part of Knights.
 *
 * Copyright (C) Stephen Thompson, 2006 - 2009.
 * Copyright (C) Kalle Marjola, 1994.
 *
 * Knights is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Knights is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Knights.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#include "misc.hpp"

#include "rng.hpp"

#include <cstdlib>
#include <ctime>
using namespace std;

// define g_rng
MersenneRNG g_rng;

void StandardRNG::seedFromTime()
{
    srand(time(0));
}

float StandardRNG::getU01()
{
    return float(rand()) / (float(RAND_MAX)+1);
}


void MersenneRNG::seedFromTime()
{
    boost::lock_guard<boost::mutex> lock(mutex);
    gen.seed(time(0));
}

float MersenneRNG::getU01()
{
    boost::lock_guard<boost::mutex> lock(mutex);
    const gen_type::result_type val = gen();
    const gen_type::result_type min = gen.min();
    const gen_type::result_type max = gen.max();
    return float(val - min) / (float(max - min) + 1.0f);
}
